%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% test_score:
%
% Calculates a pairwise marginal likelihood score.
%
% Input:
%
% - Y: the data, binary encoded as -1s and 1s
% - X_children: list of 2 containing the of latent children
% - p_G: probability of anyedge existing
% - K: maximum number of optimization iterations
% - sd_r_prior: prior standard deviation for edge-copula parameters
% - X_v: grid values shared by all latent variables
% - X_ws: cell structure containing latent weights for a range of possible 
%         correlations
% - rho_domain: possible correlation values
% - M: number of importance samples
%
% Output:
% 
% - H: bi-directed structure
% - L: loading matrix
% - rho_X: latent correlation (might not be positive definite)
%
% Created by: Ricardo Silva, London, 06/03/2012
% University College London
% Current version: 06/03/2012

function [final_score pair_scores] = ...
   test_score(Y, X_children, p_G, sd_r_prior, X_v, X_ws, rho_domain, M, ...
              L, rho_X, H)

% Initialization %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[N num_y] = size(Y);
num_x = length(X_children);
Y_parents = zeros(num_y, 1);
for x = 1:num_x
  for y = X_children{x} 
    Y_parents(y) = x;
  end
end

Y_counts = binary_sufficient(Y);

num_regions = num_x * (num_x - 1) / 2;
regions = zeros(2, num_regions); x_count = 1;
for x1 = 1:num_x    
  for x2 = (x1 + 1):num_x
    regions(:, x_count) = [x1; x2]; x_count = x_count + 1;
  end
end

[cop_theta_prior prior_w] = build_distr_approx(M, [0 sd_r_prior^2 Inf]);

scale = 25;

final_score = 0; expo_y_all = get_expo_y(1:num_y, H);
pair_scores = zeros(num_y);
for y1 = 1:num_y
  for y2 = (y1 + 1):num_y
    pair_scores(y1, y2) = ...
        score_edge_cllik_mcdn_priorbased(y1, y2, L, ...
                rho_X, rho_domain, Y_counts, H, Y_parents, expo_y_all, ...
                cop_theta_prior, prior_w, X_v, X_ws, true, scale, p_G, false); 
    pair_scores(y2, y1) = pair_scores(y1, y2);
    final_score = final_score + pair_scores(y1, y2);            
  end  
end
final_score = final_score / N;
